// Copyright HTC Corporation. All Rights Reserved.

#include "ViveOpenXRAndroidController.h"
#include "ViveOpenXRAndroidControllerUtils.h"
#include "ViveOpenXRAndroidExtensionPluginDefinition.h"

#define LOCTEXT_NAMESPACE "FViveOpenXRAndroidControllerModule"

DEFINE_LOG_CATEGORY(LogViveOpenXRAndroidController);

void FViveOpenXRAndroidControllerModule::StartupModule()
{
	CHECK_VIVEOPENXRANDROID_EXTENSION_FEATURE(wave.ControllerFocus3.enable);
	RegisterOpenXRExtensionModularFeature();

#pragma region Focus3 Keys
	EKeys::AddMenuCategoryDisplayInfo("Focus3", LOCTEXT("Focus3SubCategory", "Vive Focus3"), TEXT("GraphEditor.PadEvent_16x"));

	// "/input/squeeze/click"
	//EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Left_Grip_Click, LOCTEXT("Focus3_Left_Grip_Click", "Focus3 (L) Grip"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	//EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Right_Grip_Click, LOCTEXT("Focus3_Right_Grip_Click", "Focus3 (R) Grip"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	// "/input/squeeze/touch"
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Left_Grip_Touch, LOCTEXT("Focus3_Left_Grip_Touch", "Focus3 (L) Grip Touch"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Right_Grip_Touch, LOCTEXT("Focus3_Right_Grip_Touch", "Focus3 (R) Grip Touch"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	// "/input/squeeze/value"
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Left_Grip_Axis, LOCTEXT("Focus3_Left_Grip_Axis", "Focus3 (L) Grip Axis"), FKeyDetails::GamepadKey | FKeyDetails::Axis1D | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Right_Grip_Axis, LOCTEXT("Focus3_Right_Grip_Axis", "Focus3 (R) Grip Axis"), FKeyDetails::GamepadKey | FKeyDetails::Axis1D | FKeyDetails::NotBlueprintBindableKey, "Focus3"));

	// "/input/trigger/click"
	//EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Left_Trigger_Click, LOCTEXT("Focus3_Left_Trigger_Click", "Focus3 (L) Trigger"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	//EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Right_Trigger_Click, LOCTEXT("Focus3_Right_Trigger_Click", "Focus3 (R) Trigger"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	// "/input/trigger/touch"
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Left_Trigger_Touch, LOCTEXT("Focus3_Left_Trigger_Touch", "Focus3 (L) Trigger Touch"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Right_Trigger_Touch, LOCTEXT("Focus3_Right_Trigger_Touch", "Focus3 (R) Trigger Touch"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	// "/input/trigger/value"
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Left_Trigger_Axis, LOCTEXT("Focus3_Left_Trigger_Axis", "Focus3 (L) Trigger Axis"), FKeyDetails::GamepadKey | FKeyDetails::Axis1D | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Right_Trigger_Axis, LOCTEXT("Focus3_Right_Trigger_Axis", "Focus3 (R) Trigger Axis"), FKeyDetails::GamepadKey | FKeyDetails::Axis1D | FKeyDetails::NotBlueprintBindableKey, "Focus3"));

	// "/input/thumbstick/click"
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Left_Thumbstick_Click, LOCTEXT("Focus3_Left_Thumbstick_Click", "Focus3 (L) Thumbstick"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Right_Thumbstick_Click, LOCTEXT("Focus3_Right_Thumbstick_Click", "Focus3 (R) Thumbstick"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	// "/input/thumbstick/touch"
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Left_Thumbstick_Touch, LOCTEXT("Focus3_Left_Thumbstick_Touch", "Focus3 (L) Thumbstick Touch"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Right_Thumbstick_Touch, LOCTEXT("Focus3_Right_Thumbstick_Touch", "Focus3 (R) Thumbstick Touch"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	// "/input/thumbstick/x"
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Left_Thumbstick_X, LOCTEXT("Focus3_Left_Thumbstick_X", "Focus3 (L) Thumbstick X"), FKeyDetails::GamepadKey | FKeyDetails::Axis1D | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Right_Thumbstick_X, LOCTEXT("Focus3_Right_Thumbstick_X", "Focus3 (R) Thumbstick X"), FKeyDetails::GamepadKey | FKeyDetails::Axis1D | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	// "/input/thumbstick/y"
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Left_Thumbstick_Y, LOCTEXT("Focus3_Left_Thumbstick_Y", "Focus3 (L) Thumbstick Y"), FKeyDetails::GamepadKey | FKeyDetails::Axis1D | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Right_Thumbstick_Y, LOCTEXT("Focus3_Right_Thumbstick_Y", "Focus3 (R) Thumbstick Y"), FKeyDetails::GamepadKey | FKeyDetails::Axis1D | FKeyDetails::NotBlueprintBindableKey, "Focus3"));

	// "/input/thumbrest/touch"
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Left_Thumbrest_Touch, LOCTEXT("Focus3_Left_Thumbrest_Touch", "Focus3 (L) Thumbrest Touch"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Right_Thumbrest_Touch, LOCTEXT("Focus3_Right_Thumbrest_Touch", "Focus3 (R) Thumbrest Touch"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));

	/// ---- Left hand only ----
	// "/input/menu/click"
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Left_Menu_Click, LOCTEXT("Focus3_Left_Menu_Click", "Focus3 (L) Menu"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	// "/input/x/click"
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Left_X_Click, LOCTEXT("Focus3_Left_X_Click", "Focus3 (L) X Press"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	// "/input/y/click"
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Left_Y_Click, LOCTEXT("Focus3_Left_Y_Click", "Focus3 (L) Y Press"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));

	/// ---- Right hand only ----
	// "/input/a/click"
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Right_A_Click, LOCTEXT("Focus3_Right_A_Click", "Focus3 (R) A Press"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
	// "/input/b/click"
	EKeys::AddKey(FKeyDetails(Focus3Keys::Focus3_Right_B_Click, LOCTEXT("Focus3_Right_B_Click", "Focus3 (R) B Press"), FKeyDetails::GamepadKey | FKeyDetails::NotBlueprintBindableKey, "Focus3"));
#pragma endregion
}

void FViveOpenXRAndroidControllerModule::ShutdownModule()
{
	IModularFeatures::Get().UnregisterModularFeature(GetModularFeatureName(), this);
}


bool FViveOpenXRAndroidControllerModule::GetRequiredExtensions(TArray<const ANSICHAR*>& OutExtensions)
{
	OutExtensions.Add("XR_HTC_vive_focus3_controller_interaction");
	return true;
}

bool FViveOpenXRAndroidControllerModule::GetInteractionProfile(XrInstance InInstance, FString& OutKeyPrefix, XrPath& OutPath, bool& OutHasHaptics)
{
	OutKeyPrefix = "Focus3";
	OutHasHaptics = true;
	return xrStringToPath(InInstance, "/interaction_profiles/htc/vive_focus3_controller", &OutPath) == XR_SUCCESS;
}

#undef LOCTEXT_NAMESPACE
	
IMPLEMENT_MODULE(FViveOpenXRAndroidControllerModule, ViveOpenXRAndroidController)
