// Copyright HTC Corporation. All Rights Reserved.

#include "WaveControllerPointer.h"
#include "WaveControllerInteractInterface.h"

#include "Components/WidgetComponent.h"
#include "Engine/Engine.h"
#include "Kismet/KismetMathLibrary.h"
#include "Kismet/GameplayStatics.h"
#include "IMotionController.h"

DEFINE_LOG_CATEGORY_STATIC(LogWaveControllerPointer, Log, All);

UWaveControllerPointer::UWaveControllerPointer()
	: FadingParticle(nullptr)
	, SpotPointer(nullptr)
	, WidgetInteraction(nullptr)
	, fadingParticleSystemTemplate(nullptr)
	, fadingParticleMaterial(nullptr)
	, spotPointerMesh(nullptr)
	, spotPointerMaterial(nullptr)
	, targetDistance(TraceDistance) // Default beam length and pointer distance
{
	// Set this component to be initialized when the game starts, and to be ticked every frame.  You can turn these features
	// off to improve performance if you don't need them.
	PrimaryComponentTick.bCanEverTick = true;

	FString p_fadinglasersystem = TEXT("ParticleSystem'/ViveOpenXRAndroid/Materials/P_FadingLaserSystem.P_FadingLaserSystem'");
	fadingParticleSystemTemplate = Cast<UParticleSystem>(StaticLoadObject(UParticleSystem::StaticClass(), NULL, *p_fadinglasersystem));
	FString m_fadinglasersystem = TEXT("Material'/ViveOpenXRAndroid/Materials/M_FadingLaserSystem.M_FadingLaserSystem'");
	fadingParticleMaterial = Cast<UMaterialInterface>(StaticLoadObject(UMaterialInterface::StaticClass(), NULL, *m_fadinglasersystem));
	//FadingParticle = CreateDefaultSubobject<UParticleSystemComponent>(TEXT("FadingParticle"));
	FadingParticle = NewObject<UParticleSystemComponent>(this, TEXT("FadingParticle"));
	FadingParticle->SetupAttachment(this);

	FString reticle_mesh = TEXT("StaticMesh'/ViveOpenXRAndroid/Shapes/Reticle_Mesh.Reticle_Mesh'");
	spotPointerMesh = Cast<UStaticMesh>(StaticLoadObject(UStaticMesh::StaticClass(), NULL, *reticle_mesh));
	FString m_reticle = TEXT("Material'/ViveOpenXRAndroid/Materials/M_Reticle.M_Reticle'");
	spotPointerMaterial = Cast<UMaterialInterface>(StaticLoadObject(UMaterialInterface::StaticClass(), NULL, *m_reticle));
	//SpotPointer = CreateDefaultSubobject<UStaticMeshComponent>(TEXT("SpotPointer"));
	SpotPointer = NewObject<UStaticMeshComponent>(this, TEXT("SpotPointer"));
	SpotPointer->SetupAttachment(this);

	//WidgetInteraction = CreateDefaultSubobject<UWidgetInteractionComponent>(TEXT("WidgetInteraction"));
	WidgetInteraction = NewObject<UWidgetInteractionComponent>(this, TEXT("WidgetInteraction"));
	WidgetInteraction->SetupAttachment(this);
}

void UWaveControllerPointer::CheckLogInterval()
{
	logFrame++;
	logFrame = logFrame % klogFrameCount;
	if (logFrame == 0)
		printIntervalLog = true;
	else
		printIntervalLog = false;
}

#pragma region
void UWaveControllerPointer::SetHand(bool isLeft)
{
	IsLeft = isLeft;
	UE_LOG(LogWaveControllerPointer, Log, TEXT("IsLeft %d, SetHand()"), (uint8)IsLeft);
}
void UWaveControllerPointer::SetWidgetPointerIndex(int32 pointerIndex)
{
	if (WidgetInteraction)
	{
		WidgetPointerIndex = pointerIndex;
		WidgetInteraction->PointerIndex = WidgetPointerIndex;
		UE_LOG(LogWaveControllerPointer, Log, TEXT("m_IsLeft %d, SetWidgetPointerIndex() %f"), (uint8)m_IsLeft, WidgetInteraction->PointerIndex);
	}
}
#pragma endregion Public Interface

void UWaveControllerPointer::UpdateCustomizedVariables()
{
	if (m_IsLeft != IsLeft)
	{
		m_IsLeft = IsLeft;
		UE_LOG(LogWaveControllerPointer, Log, TEXT("m_IsLeft %d, UpdateCustomizedVariables()"), (uint8)IsLeft);
	}
	if (m_RightClickButtons != RightClickButtons || m_LeftClickButtons != LeftClickButtons)
	{
		m_RightClickButtons = RightClickButtons;
		m_LeftClickButtons = LeftClickButtons;
		for (int i = 0; i < m_RightClickButtons.Num(); i++)
			UE_LOG(LogWaveControllerPointer, Log, TEXT("UpdateCustomizedVariables() m_RightClickButtons[%d]=%s"), i, *m_RightClickButtons[i].ToString());
		for (int i = 0; i < m_LeftClickButtons.Num(); i++)
			UE_LOG(LogWaveControllerPointer, Log, TEXT("UpdateCustomizedVariables() m_LeftClickButtons[%d]=%s"), i, *m_LeftClickButtons[i].ToString());
	}
}

void UWaveControllerPointer::ConfigureResources()
{
	if (FadingParticle)
	{
		if (fadingParticleSystemTemplate)
		{
			FadingParticle->SetTemplate(fadingParticleSystemTemplate);
			UE_LOG(LogWaveControllerPointer, Log, TEXT("ConfigureResources() m_IsLeft %d, fadingParticleSystemTemplate"), (uint8)m_IsLeft);
		}
		if (fadingParticleMaterial)
		{
			FadingParticle->SetMaterial(fadingParticleMaterialIndex, fadingParticleMaterial);
			UE_LOG(LogWaveControllerPointer, Log, TEXT("ConfigureResources() m_IsLeft %d, fadingParticleMaterial"), (uint8)m_IsLeft);
		}
		FVector vec_color = UKismetMathLibrary::Conv_LinearColorToVector(FadingParticleColor);
		FadingParticle->SetVectorParameter(k_InitialColor, vec_color);
		FadingParticle->SetVectorParameter(k_InitialSize, FadingParticleSize);
	}

	if (SpotPointer)
	{
		if (spotPointerMesh)
		{
			SpotPointer->SetStaticMesh(spotPointerMesh);
			UE_LOG(LogWaveControllerPointer, Log, TEXT("ConfigureResources() m_IsLeft %d, spotPointerMesh"), (uint8)m_IsLeft);
		}
		if (spotPointerMaterial)
		{
			SpotPointer->SetMaterial(0, spotPointerMaterial);
			UE_LOG(LogWaveControllerPointer, Log, TEXT("ConfigureResources() m_IsLeft %d, spotPointerMaterial"), (uint8)m_IsLeft);
		}
		SpotPointer->SetRelativeRotation(SpotRotationOffset);
		SpotPointer->SetRelativeScale3D(SpotScale);
		SpotPointer->SetCollisionEnabled(ECollisionEnabled::Type::NoCollision);

		UE_LOG(LogWaveControllerPointer, Log, TEXT("ConfigureResources() m_IsLeft %d, SpotPointer"), (uint8)m_IsLeft);
	}

	if (WidgetInteraction)
	{
		WidgetInteraction->VirtualUserIndex = WidgetVirtualUserIndex;
		WidgetInteraction->PointerIndex = WidgetPointerIndex;
		WidgetInteraction->TraceChannel = ECollisionChannel::ECC_Visibility;
		WidgetInteraction->InteractionDistance = TraceDistance;
		WidgetInteraction->InteractionSource = EWidgetInteractionSource::World;
		//WidgetInteraction->bEnableHitTesting = true;
		//WidgetInteraction->bShowDebug = true;
		//WidgetInteraction->DebugColor = FLinearColor::Red;

		UE_LOG(LogWaveControllerPointer, Log, TEXT("ConfigureResources() WidgetInteraction m_IsLeft %d, VirtualUserIndex %d, PointerIndex %d, TraceChannel: %d, InteractionDistance: %f, InteractionSource: %d"),
			(uint8)m_IsLeft,
			WidgetInteraction->VirtualUserIndex,
			WidgetInteraction->PointerIndex,
			(uint8)WidgetInteraction->TraceChannel,
			WidgetInteraction->InteractionDistance,
			(uint8)WidgetInteraction->InteractionSource);
	}
}

void UWaveControllerPointer::SetIgnoreActors(FCollisionQueryParams& collisionParams)
{
	// Ignore the Pawn.
	for (const FWorldContext& Context : GEngine->GetWorldContexts())
	{
		APawn* pawn = UGameplayStatics::GetPlayerPawn(Context.World(), PlayerIndex);
		if (pawn)
		{
			collisionParams.AddIgnoredActor(pawn);
			//UE_LOG(LogWaveControllerPointer, Log, TEXT("SetIgnoreActors() %s"), *pawn->GetFullName());
		}
	}

	// Ignore parent actor.
	USceneComponent* parent = GetAttachParent();
	while (parent)
	{
		collisionParams.AddIgnoredActor(parent->GetOwner());
		parent = parent->GetAttachParent();
	}
}

void UWaveControllerPointer::BeginPlay()
{
	Super::BeginPlay();

	m_IsLeft = IsLeft;
	m_RightClickButtons = RightClickButtons;
	m_LeftClickButtons = LeftClickButtons;

	ConfigureResources();

	worldLocation = GetComponentLocation();
	FVector end_pos = (GetForwardVector() * TraceDistance) + worldLocation;

	if (FadingParticle)
	{
		FadingParticle->SetBeamSourcePoint(BeamEmitterIndex, worldLocation, BeamSourceIndex);
		FadingParticle->SetBeamTargetPoint(BeamEmitterIndex, end_pos, BeamSourceIndex);
	}
	if (SpotPointer)
	{
		SpotPointer->SetWorldLocation(end_pos);
	}
}

void UWaveControllerPointer::TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction)
{
	Super::TickComponent(DeltaTime, TickType, ThisTickFunction);

	CheckLogInterval();

	UpdateCustomizedVariables();

	if (!IsControllerInteractable())
		return;

	tracedPhysics = false;

	worldLocation = GetComponentLocation();
	FVector end_pos = (GetForwardVector() * TraceDistance) + worldLocation;
	worldRotation = GetComponentRotation();

	targetPoint = (GetForwardVector() * targetDistance) + worldLocation;

	UpdateButtonStates();

	//FCollisionQueryParams CollisionParams;
	//SetIgnoreActors(CollisionParams);
	//tracedPhysics = GetWorld()->LineTraceSingleByChannel(hit, worldLocation, end_pos, ECC_Camera, CollisionParams);

	FCollisionShape collision_shape;// = FCollisionShape::MakeSphere(500.0f);
	bool casted = GetWorld()->SweepMultiByChannel(outHits, worldLocation, end_pos, FQuat::Identity, ECC_WorldStatic, collision_shape);
	if (casted)
	{
		for (auto& outHit : outHits)
		{
			//AActor* hit_actor = hit.GetActor();
			AActor* hit_actor = outHit.GetActor();

			if (!hit_actor->GetClass()->ImplementsInterface(UWaveControllerInteractInterface::StaticClass()))
				continue;
			//UE_LOG(LogWaveControllerPointer, Log, TEXT("TickComponent() Hitting: %s (%s)"), *hit_actor->GetName(), *outHit.ImpactPoint.ToString());

			tracedPhysics = true;
			if (focusActor != hit_actor) // Enter & Exit
			{
				if (focusActor && (focusActor->GetClass()->ImplementsInterface(UWaveControllerInteractInterface::StaticClass())))
				{
					// Exit previous actor
					IWaveControllerInteractInterface::Execute_OnControllerExit(focusActor, GetOwner());
					UE_LOG(LogWaveControllerPointer, Log, TEXT("TickComponent() m_IsLeft %d, Exit: %s"), (uint8)m_IsLeft, *focusActor->GetName());
				}
				if (hit_actor && (hit_actor->GetClass()->ImplementsInterface(UWaveControllerInteractInterface::StaticClass())))
				{
					// Enter current actor
					IWaveControllerInteractInterface::Execute_OnControllerEnter(hit_actor, GetOwner());
					UE_LOG(LogWaveControllerPointer, Log, TEXT("TickComponent() m_IsLeft %d, Enter: %s"), (uint8)m_IsLeft, *hit_actor->GetName());
				}

				eligibleForClick = false;
				focusActor = hit_actor;
				break; // Use only the 1st hitted actor;
			}
			else // Hover & Click
			{
				if (focusActor && (focusActor->GetClass()->ImplementsInterface(UWaveControllerInteractInterface::StaticClass())))
				{
					IWaveControllerInteractInterface::Execute_OnControllerHover(focusActor, GetOwner());
					if (isDown)
					{
						IWaveControllerInteractInterface::Execute_OnControllerDown(focusActor, GetOwner());
						UE_LOG(LogWaveControllerPointer, Log, TEXT("TickComponent() m_IsLeft %d, Down: %s"), (uint8)m_IsLeft, *hit_actor->GetName());

						eligibleForClick = true;
					}
					if (isUp)
					{
						IWaveControllerInteractInterface::Execute_OnControllerUp(focusActor, GetOwner());
						UE_LOG(LogWaveControllerPointer, Log, TEXT("TickComponent() m_IsLeft %d, Up: %s"), (uint8)m_IsLeft, *hit_actor->GetName());

						if (eligibleForClick)
						{
							IWaveControllerInteractInterface::Execute_OnControllerClick(focusActor, GetOwner());
							UE_LOG(LogWaveControllerPointer, Log, TEXT("TickComponent() m_IsLeft %d, Click: %s"), (uint8)m_IsLeft, *hit_actor->GetName());
						}

						eligibleForClick = false;
					}

					// Change the beam length and pointer distance
					//targetDistance = FVector::Distance(worldLocation, hit.ImpactPoint);
					targetDistance = FVector::Distance(worldLocation, outHit.ImpactPoint);
					targetPoint = (GetForwardVector() * targetDistance) + worldLocation;
					break; // Use only the 1st hitted actor;
				}
			}
		}
	}

	if (!tracedPhysics)
	{
		if (focusActor)
		{
			if (focusActor->GetClass()->ImplementsInterface(UWaveControllerInteractInterface::StaticClass()))
			{
				// Exit previous actor
				IWaveControllerInteractInterface::Execute_OnControllerExit(focusActor, GetOwner());
				UE_LOG(LogWaveControllerPointer, Log, TEXT("TickComponent() m_IsLeft %d, Exit: %s"), (uint8)m_IsLeft, *focusActor->GetName());
			}
			focusActor = nullptr;
		}
		eligibleForClick = false;

		if (WidgetInteraction) // Click
		{
			WidgetInteraction->SetWorldLocationAndRotation(worldLocation, worldRotation);
			if (WidgetInteraction->IsOverFocusableWidget())
			{
				UWidgetComponent* widget = WidgetInteraction->GetHoveredWidgetComponent();
				UUserWidget* user_widget = widget->GetUserWidgetObject();
				tracedWidgetPosition = (static_cast<USceneComponent *>(widget))->GetComponentLocation();

				// Change the beam length and pointer distance
				targetDistance = FVector::Distance(worldLocation, tracedWidgetPosition);
				targetPoint = (GetForwardVector() * targetDistance) + worldLocation;

				if (isUp)
				{
					WidgetInteraction->PressPointerKey(FKey("LeftMouseButton"));
					WidgetInteraction->ReleasePointerKey(FKey("LeftMouseButton"));
					UE_LOG(LogWaveControllerPointer, Log, TEXT("TickComponent() m_IsLeft %d, Click widget: %s"), (uint8)m_IsLeft, *user_widget->GetFullName());
				}
			}
		}
	}

	if (FadingParticle)
	{
		FadingParticle->SetBeamSourcePoint(BeamEmitterIndex, worldLocation, BeamSourceIndex);
		FadingParticle->SetBeamTargetPoint(BeamEmitterIndex, targetPoint, BeamSourceIndex);
	}
	if (SpotPointer)
	{
		SpotPointer->SetWorldLocation(targetPoint);
		FVector pointer_scale = SpotScale;
		if (targetDistance > k_DefaultPointerDistance)
			pointer_scale = SpotScale * (targetDistance / k_DefaultPointerDistance);
		SpotPointer->SetRelativeScale3D(pointer_scale);
	}
}

bool UWaveControllerPointer::IsControllerInteractable()
{
	bool valid_pose = false;

#pragma region
	FName MotionControllerName("OpenXR");
	TArray<IMotionController*> MotionControllers = IModularFeatures::Get().GetModularFeatureImplementations<IMotionController>(IMotionController::GetModularFeatureName());
	IMotionController* MotionController = nullptr;
	for (auto Itr : MotionControllers)
	{
		if (Itr->GetMotionControllerDeviceTypeName() == MotionControllerName)
		{
			MotionController = Itr;
			break;
		}
	}

	if (MotionController)
	{
		FName AimSource = m_IsLeft ? FName("LeftAim") : FName("RightAim");
		auto trackingStatus = MotionController->GetControllerTrackingStatus(0, AimSource);
		valid_pose = (trackingStatus == ETrackingStatus::Tracked);

		if (printIntervalLog)
		{
			UE_LOG(LogWaveControllerPointer, Log, TEXT("IsControllerInteractable() m_IsLeft %d, trackingStatus %d, valid_pose %d"),
				(uint8)m_IsLeft,
				(uint8)trackingStatus,
				(uint8)valid_pose);
		}
	}
#pragma endregion Check Motion Controller

	bool interactable = valid_pose;

	if (printIntervalLog)
	{
		UE_LOG(LogWaveControllerPointer, Log, TEXT("IsControllerInteractable() m_IsLeft %d, interactable %d"), (uint8)m_IsLeft, (uint8)interactable);
	}

	if (WidgetInteraction)
	{
		if ((interactable && !WidgetInteraction->IsActive()) ||
			(!interactable && WidgetInteraction->IsActive()))
		{
			UE_LOG(LogWaveControllerPointer, Log, TEXT("IsControllerInteractable() WidgetInteraction m_IsLeft %d, interactable %d"), (uint8)m_IsLeft, (uint8)interactable);
		}
		WidgetInteraction->SetActive(interactable);
	}
	if (SpotPointer)
		SpotPointer->SetHiddenInGame(!interactable, true);
	if (FadingParticle)
		FadingParticle->SetHiddenInGame(!interactable, true);

	return interactable;
}

void UWaveControllerPointer::UpdateButtonStates()
{
	bool pressed = false;
	if (!pressed)
	{
		for (int i = 0; i < m_RightClickButtons.Num(); i++)
		{
			float keyDownTime = UGameplayStatics::GetPlayerController(GetWorld(), 0)->GetInputKeyTimeDown(m_RightClickButtons[i]);
			pressed = (keyDownTime > 0.2f);
			if (pressed)
			{
				if (!pressState) { UE_LOG(LogWaveControllerPointer, Log, TEXT("UpdateButtonStates() right %s is pressed."), *(m_RightClickButtons[i].GetFName()).ToString()); }
				break;
			}
		}
	}
	if (!pressed)
	{
		for (int i = 0; i < m_LeftClickButtons.Num(); i++)
		{
			float keyDownTime = UGameplayStatics::GetPlayerController(GetWorld(), 0)->GetInputKeyTimeDown(m_LeftClickButtons[i]);
			pressed = (keyDownTime > 0.2f);
			if (pressed)
			{
				if (!pressState) { UE_LOG(LogWaveControllerPointer, Log, TEXT("UpdateButtonStates() left %s is pressed."), *(m_LeftClickButtons[i].GetFName()).ToString()); }
				break;
			}
		}
	}

	isDown = false;
	isUp = false;

	bool down = (pressed && !pressState);
	bool hold = (pressed && pressState);
	bool up = (!pressed && pressState);

	if (!down && hold) {
		// ToDo: drag
	}
	else if (down && !eligibleForClick) {
		isDown = true;
	}
	else if (up) {
		isUp = true;
	}

	pressState = pressed;
}
