// Copyright HTC Corporation. All Rights Reserved.

#pragma once

#include "InputCoreTypes.h"

namespace Focus3Keys
{
	/**
	 * Naming rule: (upper case will be considered as lower case)
	 * Identifier (3nd word):
	 *		"grip" => <hand>/input/squeeze
	 * Component (4th word):
	 *		"axis" => <hand>/input/<Identifier>/value
	 *		"click" => <hand>/input/<Identifier>/click //if (Identifier != "trigger" && Identifier != "squeeze")
	 *		"up", "down", "left", "right" => n/a
	 *
	 * Limitation:
	 * 1. According to the naming rules, we don't support trigger/click and grip/click
	 *
	 * Issue:
	 * - If we provides trigger/click or grip/click, all input will be unavailable.
	 **/

	 // "/input/squeeze/click"
	 //const FKey Focus3_Left_Grip_Click("Focus3_Left_Grip_Click");
	 //const FKey Focus3_Right_Grip_Click("Focus3_Right_Grip_Click");
	 // "/input/squeeze/touch"
	const FKey Focus3_Left_Grip_Touch("Focus3_Left_Grip_Touch");
	const FKey Focus3_Right_Grip_Touch("Focus3_Right_Grip_Touch");
	// "/input/squeeze/value"
	const FKey Focus3_Left_Grip_Axis("Focus3_Left_Grip_Axis");
	const FKey Focus3_Right_Grip_Axis("Focus3_Right_Grip_Axis");

	// "/input/trigger/click"
	//const FKey Focus3_Left_Trigger_Click("Focus3_Left_Trigger_Click");
	//const FKey Focus3_Right_Trigger_Click("Focus3_Right_Trigger_Click");
	// "/input/trigger/touch"
	const FKey Focus3_Left_Trigger_Touch("Focus3_Left_Trigger_Touch");
	const FKey Focus3_Right_Trigger_Touch("Focus3_Right_Trigger_Touch");
	// "/input/trigger/value"
	const FKey Focus3_Left_Trigger_Axis("Focus3_Left_Trigger_Axis");
	const FKey Focus3_Right_Trigger_Axis("Focus3_Right_Trigger_Axis");

	// "/input/thumbstick/click"
	const FKey Focus3_Left_Thumbstick_Click("Focus3_Left_Thumbstick_Click");
	const FKey Focus3_Right_Thumbstick_Click("Focus3_Right_Thumbstick_Click");
	// "/input/thumbstick/touch"
	const FKey Focus3_Left_Thumbstick_Touch("Focus3_Left_Thumbstick_Touch");
	const FKey Focus3_Right_Thumbstick_Touch("Focus3_Right_Thumbstick_Touch");
	// "/input/thumbstick/x"
	const FKey Focus3_Left_Thumbstick_X("Focus3_Left_Thumbstick_X");
	const FKey Focus3_Right_Thumbstick_X("Focus3_Right_Thumbstick_X");
	// "/input/thumbstick/y"
	const FKey Focus3_Left_Thumbstick_Y("Focus3_Left_Thumbstick_Y");
	const FKey Focus3_Right_Thumbstick_Y("Focus3_Right_Thumbstick_Y");

	// "/input/thumbrest/touch"
	const FKey Focus3_Left_Thumbrest_Touch("Focus3_Left_Thumbrest_Touch");
	const FKey Focus3_Right_Thumbrest_Touch("Focus3_Right_Thumbrest_Touch");

	/// ---- Left hand only ----
	// "/input/menu/click"
	const FKey Focus3_Left_Menu_Click("Focus3_Left_Menu_Click");
	// "/input/x/click"
	const FKey Focus3_Left_X_Click("Focus3_Left_X_Click");
	// "/input/y/click"
	const FKey Focus3_Left_Y_Click("Focus3_Left_Y_Click");

	/// ---- Right hand only ----
	// "/input/a/click"
	const FKey Focus3_Right_A_Click("Focus3_Right_A_Click");
	// "/input/b/click"
	const FKey Focus3_Right_B_Click("Focus3_Right_B_Click");
}

