// Copyright HTC Corporation. All Rights Reserved.

#pragma once

#include "CoreMinimal.h"
#include "UObject/Interface.h"
#include "WaveControllerInteractInterface.generated.h"

// This class does not need to be modified.
UINTERFACE(MinimalAPI)
class UWaveControllerInteractInterface : public UInterface
{
	GENERATED_BODY()
};

class IWaveControllerInteractInterface
{
	GENERATED_BODY()

	// Add interface functions to this class. This is the class that will be inherited to implement this interface.
public:
	UFUNCTION(BlueprintCallable, BlueprintNativeEvent, Category = "Wave|OpenXR|Interaction")
	void OnControllerEnter(AActor* caller);
	UFUNCTION(BlueprintCallable, BlueprintNativeEvent, Category="Wave|OpenXR|Interaction")
	void OnControllerHover(AActor* caller);
	UFUNCTION(BlueprintCallable, BlueprintNativeEvent, Category = "Wave|OpenXR|Interaction")
	void OnControllerExit(AActor* caller);
	UFUNCTION(BlueprintCallable, BlueprintNativeEvent, Category = "Wave|OpenXR|Interaction")
	void OnControllerClick(AActor* caller);
	UFUNCTION(BlueprintCallable, BlueprintNativeEvent, Category = "Wave|OpenXR|Interaction")
	void OnControllerDown(AActor* caller);
	UFUNCTION(BlueprintCallable, BlueprintNativeEvent, Category = "Wave|OpenXR|Interaction")
	void OnControllerUp(AActor* caller);

/**
 * Usage:
 * For the "inherited" UClass object (ex: AActor):
 *
 * IWaveControllerInteractInterface* interface = Cast<IWaveControllerInteractInterface>(<ActorInstance>);
 * if (interface)
 * {
 *     interface->Execute_OnControllerHover(<ActorInstance>, GetOwner());
 * }
 *
 * For the "implemented" Blueprint object (ex: Actor):
 *
 * if (<ActorInstance>->GetClass()->ImplementsInterface(UWaveControllerInteractInterface::StaticClass()))
 * {
 *     IWaveControllerInteractInterface::Execute_OnControllerHover(<ActorInstance>, GetOwner());
 * }
 **/
};
