// Copyright HTC Corporation. All Rights Reserved.

#pragma once

#include "ViveOpenXRAndroidControllerUtils.h"

#include "CoreMinimal.h"
#include "Components/SceneComponent.h"
#include "Particles/ParticleSystemComponent.h"
#include "Particles/ParticleSystem.h"
#include "Math/Color.h"
#include "Engine/StaticMesh.h"
#include "Components/WidgetInteractionComponent.h"

#include "WaveControllerPointer.generated.h"

UCLASS( ClassGroup=(Custom), meta=(BlueprintSpawnableComponent) )
class UWaveControllerPointer : public USceneComponent
{
	GENERATED_BODY()

public:	
	UWaveControllerPointer();

protected:
	virtual void BeginPlay() override;

public:	
	virtual void TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction) override;

private:
	uint32_t logFrame = 0;
	const uint32_t klogFrameCount = 300;
	bool printIntervalLog = false;
	void CheckLogInterval();

#pragma region
protected:
	bool m_IsLeft = false;
	/** Select the device type. Note: only controller right & left take effects. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|Controller")
	bool IsLeft = false;

	/** Specify the player index of your pawn. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|Controller")
	int32 PlayerIndex = 0;

	/** Set up the available distance of controller LineTrace. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|Controller")
	float TraceDistance = 10000.f;

	const TArray<FKey> kClickButtonsLeft = { Focus3Keys::Focus3_Left_Trigger_Axis };
	TArray<FKey> m_LeftClickButtons = kClickButtonsLeft;
	/** Set up the left controller buttons for click. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|Controller")
	TArray<FKey> LeftClickButtons = kClickButtonsLeft;

	const TArray<FKey> kClickButtonsRight = { Focus3Keys::Focus3_Right_Trigger_Axis };
	TArray<FKey> m_RightClickButtons = kClickButtonsRight;
	/** Set up the right controller buttons for click. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|Controller")
	TArray<FKey> RightClickButtons = kClickButtonsRight;

	/** Set up the color of beam generated by Particle. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|Controller")
	FLinearColor FadingParticleColor = FLinearColor::Green;
	/** Set up the EmitterIndex of beam generated by Particle. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|Controller")
	int BeamEmitterIndex = 0;
	/** Set up the SourceIndex of beam generated by Particle. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|Controller")
	int BeamSourceIndex = 0;
	/** Set up the TargetIndex of beam generated by Particle. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|Controller")
	int BeamTargetIndex = 0;
	/** Set up the vector parameter of the Particle's size. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|Controller")
	FVector FadingParticleSize = FVector(0.5f, 0.5f, 0.5f);

	/** Set up the scale of the controller spot. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|Controller")
	FVector SpotScale = FVector(0.015f, 0.015f, 0.015f);
	/** Set up the rotation offset of the controller spot. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|Controller")
	FRotator SpotRotationOffset = FRotator(0, 0, 0);

	/** Set up the VirtualUserIndex of the controller widget. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|Controller")
	int32 WidgetVirtualUserIndex = 0;
	/** Set up the PointerIndex of the controller widget. Note: the Widget Pointer Index should be different in both hand. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|Controller")
	int32 WidgetPointerIndex = 1;
#pragma endregion Customized Variables

public:
	void SetHand(bool isLeft);
	void SetWidgetPointerIndex(int32 pointerIndex);

private:
	void UpdateCustomizedVariables();

private:

// Resources
protected:
	UParticleSystemComponent* FadingParticle;
	UStaticMeshComponent* SpotPointer;
	UWidgetInteractionComponent* WidgetInteraction;

private:
	void ConfigureResources();
	bool IsControllerInteractable();

private:
	UParticleSystem* fadingParticleSystemTemplate;
	const int fadingParticleMaterialIndex = 0;
	UMaterialInterface* fadingParticleMaterial;
	const FName k_InitialColor = FName(TEXT("ParticleInitialColor"));
	const FName k_InitialSize = FName(TEXT("ParticleInitialSize"));

	UStaticMesh* spotPointerMesh;
	UMaterialInterface* spotPointerMaterial;

// Trace
private:
	FVector worldLocation = FVector::ZeroVector;
	FRotator worldRotation = FRotator::ZeroRotator;

	void SetIgnoreActors(FCollisionQueryParams& collisionParams);

	FVector targetPoint = FVector::ZeroVector;

	bool tracedPhysics = false;
	//FHitResult hit;
	TArray<FHitResult> outHits;
	AActor* focusActor;
	const float k_DefaultPointerDistance = 100;
	float targetDistance;

	FVector tracedWidgetPosition;

	FVector devicePosition = FVector::ZeroVector;
	FRotator deviceRotation = FRotator::ZeroRotator;

// Button
private:
	bool pressState;
	bool eligibleForClick = false;
	bool isDown = false, isUp = false;
	void UpdateButtonStates();
};
