// Copyright HTC Corporation. All Rights Reserved.

#include "ISettingsModule.h"
#include "ISettingsSection.h"
#include "ViveOpenXRAndroidSettings.h"

#define LOCTEXT_NAMESPACE "ViveOpenXRAndroidSettings"

class FViveOpenXRAndroidSettingsModule : public IModuleInterface
{
	virtual void StartupModule() override
	{
		RegisterSettings();
	}

	virtual void ShutdownModule() override
	{
		if (UObjectInitialized())
		{
			UnregisterSettings();
		}
	}

private:
	// Callback for when the settings were saved.
	bool HandleSettingsSaved()
	{
		UViveOpenXRAndroidSettings* Settings = GetMutableDefault<UViveOpenXRAndroidSettings>();
		bool isValidate = false;
		// Do validate here
		isValidate = true;

		if (isValidate)
			Settings->SaveConfig();

		return true;
	}

	void RegisterSettings()
	{
		if (ISettingsModule* SettingsModule = FModuleManager::GetModulePtr<ISettingsModule>("Settings"))
		{
			// Register the settings
			ISettingsSectionPtr SettingsSection = SettingsModule->RegisterSettings("Project", "Plugins", "ViveOpenXRAndroid",
				LOCTEXT("RuntimeGeneralSettingsName", "VIVE OpenXR - Android"),
				LOCTEXT("RuntimeGeneralSettingsDescription", "Configurations for VIVE OpenXR Android Extensions."),
				GetMutableDefault<UViveOpenXRAndroidSettings>()
			);

			// Register the save handler to your settings, you might want to use it to
			// validate those or just act to settings changes.
			if (SettingsSection.IsValid())
			{
				SettingsSection->OnModified().BindRaw(this, &FViveOpenXRAndroidSettingsModule::HandleSettingsSaved);
			}
		}
	}

	void UnregisterSettings()
	{
		// Ensure to unregister all of your registered settings here, hot-reload would
		// otherwise yield unexpected results.

		if (ISettingsModule* SettingsModule = FModuleManager::GetModulePtr<ISettingsModule>("Settings"))
		{
			SettingsModule->UnregisterSettings("Project", "Plugins", "ViveOpenXRAndroid");
		}
	}
};

IMPLEMENT_MODULE(FViveOpenXRAndroidSettingsModule, ViveOpenXRAndroidEditor);

#undef LOCTEXT_NAMESPACE
