# 12.68. XR_HTC_facial_tracking
## Name String
    XR_HTC_facial_tracking
## Revision
    1
## Overview
This extension allows an application to track and integrate users' eye and lip movements, empowering developers to read intention and model facial expressions.
## Eye Blend Shapes
Through feeding the blend shape values of eye expression to an avatar, its facial expression can be animated with the players eye movement. The following pictures show how the facial expression acts on the avatar according to each set of eye blend shape values.

    typedef enum XrEyeExpressionHTC {
        XR_EYE_EXPRESSION_LEFT_BLINK_HTC = 0,
        XR_EYE_EXPRESSION_LEFT_WIDE_HTC = 1,
        XR_EYE_EXPRESSION_RIGHT_BLINK_HTC = 2,
        XR_EYE_EXPRESSION_RIGHT_WIDE_HTC = 3,
        XR_EYE_EXPRESSION_LEFT_SQUEEZE_HTC = 4,
        XR_EYE_EXPRESSION_RIGHT_SQUEEZE_HTC = 5,
        XR_EYE_EXPRESSION_LEFT_DOWN_HTC = 6,
        XR_EYE_EXPRESSION_RIGHT_DOWN_HTC = 7,
        XR_EYE_EXPRESSION_LEFT_OUT_HTC = 8,
        XR_EYE_EXPRESSION_RIGHT_IN_HTC = 9,
        XR_EYE_EXPRESSION_LEFT_IN_HTC = 10,
        XR_EYE_EXPRESSION_RIGHT_OUT_HTC = 11,
        XR_EYE_EXPRESSION_LEFT_UP_HTC = 12,
        XR_EYE_EXPRESSION_RIGHT_UP_HTC = 13,
        XR_EYE_EXPRESSION_MAX_ENUM_HTC
    } XrEyeExpressionHTC;
## Lip Blend Shapes
Through feeding the blend shape values of lip expression to an avatar, its facial expression can be animated with the players lip movement. The following pictures show how the facial expression acts on the avatar according to each set of lip blend shape values.

    typedef enum XrLipExpressionHTC {
        XR_LIP_EXPRESSION_JAW_RIGHT_HTC = 0,
        XR_LIP_EXPRESSION_JAW_LEFT_HTC = 1,
        XR_LIP_EXPRESSION_JAW_FORWARD_HTC = 2,
        XR_LIP_EXPRESSION_JAW_OPEN_HTC = 3,
        XR_LIP_EXPRESSION_MOUTH_APE_SHAPE_HTC = 4,
        XR_LIP_EXPRESSION_MOUTH_UPPER_RIGHT_HTC = 5,
        XR_LIP_EXPRESSION_MOUTH_UPPER_LEFT_HTC = 6,
        XR_LIP_EXPRESSION_MOUTH_LOWER_RIGHT_HTC = 7,
        XR_LIP_EXPRESSION_MOUTH_LOWER_LEFT_HTC = 8,
        XR_LIP_EXPRESSION_MOUTH_UPPER_OVERTURN_HTC = 9,
        XR_LIP_EXPRESSION_MOUTH_LOWER_OVERTURN_HTC = 10,
        XR_LIP_EXPRESSION_MOUTH_POUT_HTC = 11,
        XR_LIP_EXPRESSION_MOUTH_SMILE_RIGHT_HTC = 12,
        XR_LIP_EXPRESSION_MOUTH_SMILE_LEFT_HTC = 13,
        XR_LIP_EXPRESSION_MOUTH_SAD_RIGHT_HTC = 14,
        XR_LIP_EXPRESSION_MOUTH_SAD_LEFT_HTC = 15,
        XR_LIP_EXPRESSION_CHEEK_PUFF_RIGHT_HTC = 16,
        XR_LIP_EXPRESSION_CHEEK_PUFF_LEFT_HTC = 17,
        XR_LIP_EXPRESSION_CHEEK_SUCK_HTC = 18,
        XR_LIP_EXPRESSION_MOUTH_UPPER_UPRIGHT_HTC = 19,
        XR_LIP_EXPRESSION_MOUTH_UPPER_UPLEFT_HTC = 20,
        XR_LIP_EXPRESSION_MOUTH_LOWER_DOWNRIGHT_HTC = 21,
        XR_LIP_EXPRESSION_MOUTH_LOWER_DOWNLEFT_HTC = 22,
        XR_LIP_EXPRESSION_MOUTH_UPPER_INSIDE_HTC = 23,
        XR_LIP_EXPRESSION_MOUTH_LOWER_INSIDE_HTC = 24,
        XR_LIP_EXPRESSION_MOUTH_LOWER_OVERLAY_HTC = 25,
        XR_LIP_EXPRESSION_TONGUE_LONGSTEP1_HTC = 26,
        XR_LIP_EXPRESSION_TONGUE_LEFT_HTC = 27,
        XR_LIP_EXPRESSION_TONGUE_RIGHT_HTC = 28,
        XR_LIP_EXPRESSION_TONGUE_UP_HTC = 29,
        XR_LIP_EXPRESSION_TONGUE_DOWN_HTC = 30,
        XR_LIP_EXPRESSION_TONGUE_ROLL_HTC = 31,
        XR_LIP_EXPRESSION_TONGUE_LONGSTEP2_HTC = 32,
        XR_LIP_EXPRESSION_TONGUE_UPRIGHT_MORPH_HTC = 33,
        XR_LIP_EXPRESSION_TONGUE_UPLEFT_MORPH_HTC = 34,
        XR_LIP_EXPRESSION_TONGUE_DOWNRIGHT_MORPH_HTC = 35,
        XR_LIP_EXPRESSION_TONGUE_DOWNLEFT_MORPH_HTC = 36,
        XR_LIP_EXPRESSION_MAX_ENUM_HTC
    } XrLipExpressionHTC;

## Wave Blueprint Function Library

    UENUM(BlueprintType, Category = "ViveOpenXRAndroid|OpenXR|FacialTracking")
    enum class EXrFacialTrackingType : uint8 {
    	None = 0  UMETA(Hidden),
    	Eye = 1,
    	Lip = 2
    };
    
    /** The avatar's eye relative blend shape.*/
    UENUM(BlueprintType, Category = "ViveOpenXRAndroid|OpenXR|FacialTracking")
    enum class EEyeShape :uint8 {
    	Eye_Left_Blink = 0	 UMETA(DisplayName = "Eye_Left_Blink"),
    	Eye_Left_Wide = 1	 UMETA(DisplayName = "Eye_Left_Wide"),
    	Eye_Left_Right = 2	 UMETA(DisplayName = "Eye_Left_Right"),
    	Eye_Left_Left = 3	 UMETA(DisplayName = "Eye_Left_Left"),
    	Eye_Left_Up = 4		 UMETA(DisplayName = "Eye_Left_Up"),
    	Eye_Left_Down = 5	 UMETA(DisplayName = "Eye_Left_Down"),
    	Eye_Right_Blink = 6	 UMETA(DisplayName = "Eye_Right_Blink"),
    	Eye_Right_Wide = 7	 UMETA(DisplayName = "Eye_Right_Wide"),
    	Eye_Right_Right = 8	 UMETA(DisplayName = "Eye_Right_Right"),
    	Eye_Right_Left = 9	 UMETA(DisplayName = "Eye_Right_Left"),
    	Eye_Right_Up = 10	 UMETA(DisplayName = "Eye_Right_Up"),
    	Eye_Right_Down = 11  UMETA(DisplayName = "Eye_Right_Down"),
    	Eye_Frown = 12		 UMETA(DisplayName = "Eye_Frown"),
    	Eye_Left_Squeeze = 13  UMETA(DisplayName = "Eye_Left_Squeeze"),
    	Eye_Right_Squeeze = 14 UMETA(DisplayName = "Eye_Right_Squeeze"),
    	Max = 15			   UMETA(Hidden),
    	None = 63			   UMETA(DisplayName = "None"),
    };
    
    /** The prediction result relative blend shape.*/
    UENUM(BlueprintType, Category = "ViveOpenXRAndroid|OpenXR|FacialTracking")
    enum class ELipShape :uint8 {
      Jaw_Right = 0                UMETA(DisplayName = "Jaw_Right"),
      Jaw_Left = 1                 UMETA(DisplayName = "Jaw_Left"),
      Jaw_Forward = 2              UMETA(DisplayName = "Jaw_Forward"),
      Jaw_Open = 3                 UMETA(DisplayName = "Jaw_Open"),
      Mouth_Ape_Shape = 4          UMETA(DisplayName = "Mouth_Ape_Shape"),
      Mouth_Upper_Right = 5        UMETA(DisplayName = "Mouth_Upper_Right"),
      Mouth_Upper_Left = 6         UMETA(DisplayName = "Mouth_Upper_Left"),
      Mouth_Lower_Right = 7        UMETA(DisplayName = "Mouth_Lower_Right"),
      Mouth_Lower_Left = 8         UMETA(DisplayName = "Mouth_Lower_Left"),
      Mouth_Upper_Overturn = 9     UMETA(DisplayName = "Mouth_Upper_Overturn"),
      Mouth_Lower_Overturn = 10    UMETA(DisplayName = "Mouth_Lower_Overturn"),
      Mouth_Pout = 11              UMETA(DisplayName = "Mouth_Pout"),
      Mouth_Smile_Right = 12       UMETA(DisplayName = "Mouth_Smile_Right"),
      Mouth_Smile_Left = 13        UMETA(DisplayName = "Mouth_Smile_Left"),
      Mouth_Sad_Right = 14         UMETA(DisplayName = "Mouth_Sad_Right"),
      Mouth_Sad_Left = 15          UMETA(DisplayName = "Mouth_Sad_Left"),
      Cheek_Puff_Right = 16        UMETA(DisplayName = "Cheek_Puff_Right"),
      Cheek_Puff_Left = 17         UMETA(DisplayName = "Cheek_Puff_Left"),
      Cheek_Suck = 18              UMETA(DisplayName = "Cheek_Suck"),
      Mouth_Upper_UpRight = 19     UMETA(DisplayName = "Mouth_Upper_UpRight"),
      Mouth_Upper_UpLeft = 20      UMETA(DisplayName = "Mouth_Upper_UpLeft"),
      Mouth_Lower_DownRight = 21   UMETA(DisplayName = "Mouth_Lower_DownRight"),
      Mouth_Lower_DownLeft = 22    UMETA(DisplayName = "Mouth_Lower_DownLeft"),
      Mouth_Upper_Inside = 23      UMETA(DisplayName = "Mouth_Upper_Inside"),
      Mouth_Lower_Inside = 24      UMETA(DisplayName = "Mouth_Lower_Inside"),
      Mouth_Lower_Overlay = 25     UMETA(DisplayName = "Mouth_Lower_Overlay"),
      Tongue_LongStep1 = 26        UMETA(DisplayName = "Tongue_LongStep1"),
      Tongue_Left = 27             UMETA(DisplayName = "Tongue_Left"),
      Tongue_Right = 28            UMETA(DisplayName = "Tongue_Right"),
      Tongue_Up = 29               UMETA(DisplayName = "Tongue_Up"),
      Tongue_Down = 30             UMETA(DisplayName = "Tongue_Down"),
      Tongue_Roll = 31             UMETA(DisplayName = "Tongue_Roll"),
      Tongue_LongStep2 = 32        UMETA(DisplayName = "Tongue_LongStep2"),
      Tongue_UpRight_Morph = 33    UMETA(DisplayName = "Tongue_UpRight_Morph"),
      Tongue_UpLeft_Morph = 34     UMETA(DisplayName = "Tongue_UpLeft_Morph"),
      Tongue_DownRight_Morph = 35  UMETA(DisplayName = "Tongue_DownRight_Morph"),
      Tongue_DownLeft_Morph = 36   UMETA(DisplayName = "Tongue_DownLeft_Morph"),
      Max = 37                     UMETA(Hidden),
      None = 63                    UMETA(DisplayName = "None"),
    };

    void UViveOpenXRAndroidFacialTrackingFunctionLibrary::GetIsFacialTrackingEnabled(bool& result)
    bool UViveOpenXRAndroidFacialTrackingFunctionLibrary::CreateFacialTracker(EXrFacialTrackingType trackingType)
    bool UViveOpenXRAndroidFacialTrackingFunctionLibrary::DestroyFacialTracker()
    bool UViveOpenXRAndroidFacialTrackingFunctionLibrary::GetEyeFacialExpressions(bool& isActive, TMap<EEyeShape, float>& blendshapes)
    bool UViveOpenXRAndroidFacialTrackingFunctionLibrary::GetLipFacialExpressions(bool& isActive, TMap<ELipShape, float>& blendshapes)
