﻿// Copyright (c) 2022 HTC Corporation. All Rights Reserved.

#include "LiveLinkOpenXRFacialTrackingSourceFactory.h"
#include "IViveOpenXRAndroidFacialTrackingModule.h"

#include "Features/IModularFeatures.h"
#include "ILiveLinkClient.h"

#define LOCTEXT_NAMESPACE "ViveOpenXRAndroidFacialTracking"

FText ULiveLinkOpenXRFacialTrackingSourceFactory::GetSourceDisplayName() const
{
	return LOCTEXT("FacialTrackingLiveLinkSourceName", "Vive OpenXR Facial Tracking Source");
}

FText ULiveLinkOpenXRFacialTrackingSourceFactory::GetSourceTooltip() const
{
	return LOCTEXT("FacialTrackingLiveLinkSourceTooltip", "Vive OpenXR Facial Tracking Key Points Source");
}

ULiveLinkOpenXRFacialTrackingSourceFactory::EMenuType ULiveLinkOpenXRFacialTrackingSourceFactory::GetMenuType() const
{
	if (IModularFeatures::Get().IsModularFeatureAvailable(ILiveLinkClient::ModularFeatureName))
	{
		ILiveLinkClient& LiveLinkClient = IModularFeatures::Get().GetModularFeature<ILiveLinkClient>(ILiveLinkClient::ModularFeatureName);

		if (!IViveOpenXRAndroidFacialTrackingModule::Get().IsLiveLinkSourceValid() || !LiveLinkClient.HasSourceBeenAdded(IViveOpenXRAndroidFacialTrackingModule::Get().GetLiveLinkSource()))
		{
			return EMenuType::MenuEntry;
		}
	}
	return EMenuType::Disabled;
}

TSharedPtr<ILiveLinkSource> ULiveLinkOpenXRFacialTrackingSourceFactory::CreateSource(const FString& ConnectionString) const
{
	return IViveOpenXRAndroidFacialTrackingModule::Get().GetLiveLinkSource();
}

#undef LOCTEXT_NAMESPACE