// Copyright (c) 2022 HTC Corporation. All Rights Reserved.

#pragma once

#include "CoreMinimal.h"
#include "Components/SkeletalMeshComponent.h"
#include "GameFramework/Actor.h"
#include "ViveFacialExpressionEnums.h"									// Get facial experssion enums from ViveOpenXR plugin.
#include "ViveOpenXRAndroidFacialTrackingFunctionLibrary.h"		// These provide the functionality required for ViveOpenXR plugin - Facial Tracking.
#include "FT_AvatarSample.generated.h"

UCLASS()
class AFT_AvatarSample : public AActor
{
	GENERATED_BODY()
	
public:	
	// Sets default values for this actor's properties
	AFT_AvatarSample();

protected:
	// Called when the game starts or when spawned
	virtual void BeginPlay() override;
	virtual void EndPlay(const EEndPlayReason::Type EndPlayReason) override;

public:	
	// Called every frame
	virtual void Tick(float DeltaTime) override;

	/** Avatar head mesh. */
	UPROPERTY(VisibleDefaultsOnly, Category = Mesh)
	USkeletalMeshComponent* HeadModel;
	/** Avatar left eye mesh. */
	UPROPERTY(VisibleDefaultsOnly, Category = Mesh)
	USkeletalMeshComponent* EyeModel_L;
	/** Avatar right eye mesh. */
	UPROPERTY(VisibleDefaultsOnly, Category = Mesh)
	USkeletalMeshComponent* EyeModel_R;
	/** Left eye's anchor. */
	UPROPERTY(VisibleDefaultsOnly, Category = Anchor)
	USceneComponent* EyeAnchor_L;
	/** Right eye's anchor. */
	UPROPERTY(VisibleDefaultsOnly, Category = Anchor)
	USceneComponent* EyeAnchor_R;

	/** If true, this Actor will update Eye expressions every frame. */
	UPROPERTY(EditAnywhere, Category = FacialTrackingSettings)
	bool EnableEye = true;
	/** If true, this Actor will update Lip expressions every frame. */
	UPROPERTY(EditAnywhere, Category = FacialTrackingSettings)
	bool EnableLip = true;

	bool isEyeActive = false;
	bool isLipActive = false;
private:
	/** ­We will use this variable to set both eyes anchor. */
	TArray<USceneComponent*> EyeAnchors;
	/** ­We will use this variable to set eye expressions weighting. */
	TMap<EEyeShape, float> EyeWeighting;
	/** ­We will use this variable to set lip expressions weighting. */
	TMap<ELipShape, float> LipWeighting;
	
	/** ­This TMap variable is used to store the corresponding result of Avatar's eye blend shapes(key)
			and OpenXRFacialTracking eye expressions(value). */
	TMap<FName, EEyeShape> EyeShapeTable;
	/** ­This TMap variable is used to store the corresponding result of Avatar's lip blend shapes(key)
			and OpenXRFacialTracking lip expressions(value). */
	TMap<FName, ELipShape> LipShapeTable;

	/** This function will calculate the gaze direction  to update the eye's anchors rotation
			to represent the direction of the eye's gaze. */
	void UpdateGazeRay();

	FORCEINLINE FVector ConvetToUnrealVector(FVector Vector, float Scale = 1.0f)
	{
		return FVector(Vector.Z * Scale, Vector.X * Scale, Vector.Y * Scale);
	}

	/** Render the result of face tracking to the avatar's blend shapes. */
	template<typename T>
	void RenderModelShape(USkeletalMeshComponent* model, TMap<FName, T> shapeTable, TMap<T, float> weighting);
};
