// Copyright HTC Corporation. All Rights Reserved.

#include "ViveOpenXRAndroidHMD.h"
#include "OpenXRCore.h"
#include "OpenXRPlatformRHI.h"
#include "DefaultSpectatorScreenController.h"

#if PLATFORM_ANDROID
#include "Android/AndroidPlatformMisc.h"
#include <dlfcn.h>
#endif

DEFINE_LOG_CATEGORY(LogViveOpenXRAndroidHMD);

bool FViveOpenXRAndroidHMD::GetCustomLoader(PFN_xrGetInstanceProcAddr* OutGetProcAddr)
{
#if PLATFORM_ANDROID
	UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("GetCustomLoader"));

	dlerror();  // clear errors

	LoaderHandle = FPlatformProcess::GetDllHandle(TEXT("libopenxr_loader.so"));
	if (LoaderHandle == nullptr)
	{
		UE_LOG(LogViveOpenXRAndroidHMD, Error, TEXT("Unable to load libopenxr_loader.so, error %s"), ANSI_TO_TCHAR(dlerror()));
		return false;
	}

	dlerror();  // clear errors

	PFN_xrGetInstanceProcAddr xrGetInstanceProcAddrPtr = (PFN_xrGetInstanceProcAddr)FPlatformProcess::GetDllExport(LoaderHandle, TEXT("xrGetInstanceProcAddr"));
	if (xrGetInstanceProcAddrPtr == nullptr)
	{
		UE_LOG(LogViveOpenXRAndroidHMD, Error, TEXT("Unable to load OpenXR xrGetInstanceProcAddr, error %s"), ANSI_TO_TCHAR(dlerror()));
		return false;
	}
	*OutGetProcAddr = xrGetInstanceProcAddrPtr;

	extern struct android_app* GNativeAndroidApp;
	PFN_xrInitializeLoaderKHR xrInitializeLoaderKHR;
	xrGetInstanceProcAddrPtr(XR_NULL_HANDLE, "xrInitializeLoaderKHR", (PFN_xrVoidFunction*)&xrInitializeLoaderKHR);
	if (xrInitializeLoaderKHR == nullptr)
	{
		UE_LOG(LogViveOpenXRAndroidHMD, Error, TEXT("Unable to load OpenXR xrInitializeLoaderKHR"));
		return false;
	}
	XrLoaderInitInfoAndroidKHR LoaderInitializeInfoAndroid;
	LoaderInitializeInfoAndroid.type = XR_TYPE_LOADER_INIT_INFO_ANDROID_KHR;
	LoaderInitializeInfoAndroid.next = NULL;
	LoaderInitializeInfoAndroid.applicationVM = GNativeAndroidApp->activity->vm;
	LoaderInitializeInfoAndroid.applicationContext = GNativeAndroidApp->activity->clazz;
	XR_ENSURE(xrInitializeLoaderKHR((XrLoaderInitInfoBaseHeaderKHR*)&LoaderInitializeInfoAndroid));

	UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Loaded Wave OpenXR Loader"));
	return true;
#else
	return false;
#endif
}

bool FViveOpenXRAndroidHMD::IsStandaloneStereoOnlyDevice()
{
#if PLATFORM_ANDROID
	return true;
#else
	return false;
#endif
}

bool FViveOpenXRAndroidHMD::GetRequiredExtensions(TArray<const ANSICHAR*>& OutExtensions)
{
	UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR GetRequiredExtensions"));
	return true;
}

bool FViveOpenXRAndroidHMD::GetInteractionProfile(XrInstance InInstance, FString& OutKeyPrefix, XrPath& OutPath, bool& OutHasHaptics)
{
	UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR GetInteractionProfile"));
	return true;
}

bool FViveOpenXRAndroidHMD::GetSpectatorScreenController(FHeadMountedDisplayBase* InHMDBase, TUniquePtr<FDefaultSpectatorScreenController>& OutSpectatorScreenController)
{
	UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR GetSpectatorScreenController"));
#if PLATFORM_ANDROID
	OutSpectatorScreenController = nullptr;
	return true;
#else
	OutSpectatorScreenController = MakeUnique<FDefaultSpectatorScreenController>(InHMDBase);
	return false;
#endif
}

void FViveOpenXRAndroidHMD::AddActions(XrInstance Instance, TFunction<XrAction(XrActionType InActionType, const FName& InName, const TArray<XrPath>& InSubactionPaths)> AddAction)
{
	UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR AddActions"));
	return;
}

void FViveOpenXRAndroidHMD::OnEvent(XrSession InSession, const XrEventDataBaseHeader* InHeader)
{
	UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR OnEvent"));
	return;
}

const void* FViveOpenXRAndroidHMD::OnCreateInstance(class IOpenXRHMDPlugin* InPlugin, const void* InNext)
{
	UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR OnCreateInstance"));
	return InNext;
}

const void* FViveOpenXRAndroidHMD::OnGetSystem(XrInstance InInstance, const void* InNext)
{
	UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR OnGetSystem"));
	return InNext;
}

const void* FViveOpenXRAndroidHMD::OnCreateSession(XrInstance InInstance, XrSystemId InSystem, const void* InNext)
{
	UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR OnCreateSession"));
#if PLATFORM_ANDROID
	if (GRHISupportsRHIThread && GIsThreadedRendering && GUseRHIThread_InternalUseOnly)
	{
		SetRHIThreadEnabled(false, false);
	}
#endif
	return InNext;
}

const void* FViveOpenXRAndroidHMD::OnBeginSession(XrSession InSession, const void* InNext)
{
	UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR OnBeginSession"));
	return InNext;
}

const void* FViveOpenXRAndroidHMD::OnBeginFrame(XrSession InSession, XrTime DisplayTime, const void* InNext)
{
	//UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR OnBeginFrame"));
	return InNext;
}

const void* FViveOpenXRAndroidHMD::OnBeginProjectionView(XrSession InSession, int32 InLayerIndex, int32 InViewIndex, const void* InNext)
{
	//UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR OnBeginProjectionView"));
	return InNext;
}

const void* FViveOpenXRAndroidHMD::OnBeginDepthInfo(XrSession InSession, int32 InLayerIndex, int32 InViewIndex, const void* InNext)
{
	UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR OnBeginDepthInfo"));
	return InNext;
}

const void* FViveOpenXRAndroidHMD::OnEndProjectionLayer(XrSession InSession, int32 InLayerIndex, const void* InNext, XrCompositionLayerFlags& OutFlags)
{
	//UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR OnEndProjectionLayer"));
	OutFlags |= XR_COMPOSITION_LAYER_CORRECT_CHROMATIC_ABERRATION_BIT;
	return InNext;
}

const void* FViveOpenXRAndroidHMD::OnEndFrame(XrSession InSession, XrTime DisplayTime, const TArray<XrSwapchainSubImage> InColorImages, const TArray<XrSwapchainSubImage> InDepthImages, const void* InNext)
{
	//UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR OnEndFrame"));
	return InNext;
}

const void* FViveOpenXRAndroidHMD::OnSyncActions(XrSession InSession, const void* InNext)
{
	//UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR OnSyncActions"));
	return InNext;
}

void FViveOpenXRAndroidHMD::PostSyncActions(XrSession InSession)
{
	//UE_LOG(LogViveOpenXRAndroidHMD, Log, TEXT("Wave OpenXR PostSyncActions"));
	return;
}

IMPLEMENT_MODULE(FViveOpenXRAndroidHMD, ViveOpenXRAndroidHMD)
