// Copyright HTC Corporation. All Rights Reserved.

/**
 * To Enable/Disable the ViveOpenXRAndroid Extension Feature by project settings.
 * How to:
 *	1. Register your feature in project settings:
 *		1) Costumize your CVar string(such as wave.HandTracking.enable) and add your CVAr in ViveOpenXRAndroidHMD/Private/ViveOpenXRAndroidConsoleVariable.cpp
 *		2) Register your CVar string in ViveOpenXRAndroidEditor(ViveOpenXRAndroidSettings.h and ViveOpenXRAndroidSettings.cpp).
 *		3) Please keep the initial(default) values above identical.
 *	2. Add MACRO CHECK_VIVEOPENXRANDROID_EXTENSION_FEATURE(FeatureCVarString); in StartupModule() which you override from IModuleInterface
 *	Such as:
 *		virtual void StartupModule() override
 *		{
 *			CHECK_VIVEOPENXRANDROID_EXTENSION_FEATURE(wave.HandTracking.enable);
 *			IModularFeatures::Get().RegisterModularFeature(GetModularFeatureName(), this);
 *		}
 *
 *		Or,
 *
 *		virtual void StartupModule() override
 *		{
 *			CHECK_VIVEOPENXRANDROID_EXTENSION_FEATURE(wave.HandTracking.enable);
 *			RegisterOpenXRExtensionModularFeature();
 *		}
 *
 *	The module of the feature won't register because CHECK_VIVEOPENXRANDROID_EXTENSION_FEATURE will return directly if the feature is disabled by project settings.
 */

#ifndef ViveOpenXRAndroidExtensionPluginDefinition_h_
#define ViveOpenXRAndroidExtensionPluginDefinition_h_

#define CHECK_VIVEOPENXRANDROID_EXTENSION_FEATURE(FeatureCVarString) \
	FString FeatureConsoleVariable = FString(#FeatureCVarString); \
	ApplyCVarSettingsFromIni(TEXT("/Script/ViveOpenXRAndroidEditor.ViveOpenXRAndroidSettings"), *GEngineIni, ECVF_SetByProjectSetting); \
	auto* CVar = IConsoleManager::Get().FindConsoleVariable(*FeatureConsoleVariable); \
	if (CVar) { \
		if(!CVar->GetBool()) return; \
	} else { \
		FString LogFeatureText = FString("The CVar string ") + FeatureConsoleVariable + FString(" is invalid, please check the typo of CVar string."); \
		UE_LOG(LogTemp, Fatal, TEXT("%s"), *LogFeatureText); \
		return; \
	}

#endif //ViveOpenXRAndroidExtensionPluginDefinition_h_
