// Copyright HTC Corporation. All Rights Reserved.

#include "ViveOpenXRAndroidHandInteractionBP.h"
#include "ViveOpenXRAndroidHandInteraction.h"

TSharedPtr< class FViveOpenXRAndroidHandInteraction, ESPMode::ThreadSafe > GetHandInteraction()
{
	TArray<FViveOpenXRAndroidHandInteractionModule*> HandInteractions = IModularFeatures::Get().GetModularFeatureImplementations<FViveOpenXRAndroidHandInteractionModule>(FViveOpenXRAndroidHandInteractionModule::GetModularFeatureName());
	for (auto HandInteraction : HandInteractions)
	{
		return HandInteraction->CreateHandInteraction();
	}

	return nullptr;
}

float UViveOpenXRAndroidHandInteractionBP::GetPinchStrength(bool isLeft)
{
	auto HandInteraction = GetHandInteraction();
	if (HandInteraction != nullptr)
		return HandInteraction->GetPinchStrength(isLeft);

	return 0;
}

FQuat UViveOpenXRAndroidHandInteractionBP::GetPinchRotation(bool isLeft)
{
	auto HandInteraction = GetHandInteraction();
	if (HandInteraction != nullptr)
		return HandInteraction->GetPinchRotation(isLeft);

	return FQuat::Identity;
}

FVector UViveOpenXRAndroidHandInteractionBP::GetPinchPosition(bool isLeft)
{
	auto HandInteraction = GetHandInteraction();
	if (HandInteraction != nullptr)
		return HandInteraction->GetPinchPosition(isLeft);

	return FVector::ZeroVector;
}

bool UViveOpenXRAndroidHandInteractionBP::GetPinchActive(bool isLeft)
{
	auto HandInteraction = GetHandInteraction();
	if (HandInteraction != nullptr)
		return HandInteraction->GetPinchActive(isLeft);

	return false;
}

bool UViveOpenXRAndroidHandInteractionBP::GetPinchValid(bool isLeft)
{
	auto HandInteraction = GetHandInteraction();
	if (HandInteraction != nullptr)
		return HandInteraction->GetPinchValid(isLeft);

	return false;
}

bool UViveOpenXRAndroidHandInteractionBP::GetPinchTracked(bool isLeft)
{
	auto HandInteraction = GetHandInteraction();
	if (HandInteraction != nullptr)
		return HandInteraction->GetPinchTracked(isLeft);

	return false;
}
