// Copyright HTC Corporation. All Rights Reserved.

#include "WaveHandPointer.h"
#include "ViveOpenXRAndroidHandInteractionBP.h"
#include "WaveHandInteractInterface.h"

#include "Components/WidgetComponent.h"
#include "Kismet/KismetMathLibrary.h"
#include "Kismet/GameplayStatics.h"
#include "DrawDebugHelpers.h"

DEFINE_LOG_CATEGORY_STATIC(LogWaveHandPointer, Log, All);

UWaveHandPointer::UWaveHandPointer()
	: FadingParticle(nullptr)
	, SpotPointer(nullptr)
	, SpotPointerBlue(nullptr)
	, WidgetInteraction(nullptr)
	, fadingParticleSystemTemplate(nullptr)
	, fadingParticleMaterial(nullptr)
	, spotPointerMesh(nullptr)
	, spotPointerMaterial(nullptr)
	, spotPointerBlueMesh(nullptr)
	, spotPointerBlueMaterial(nullptr)
	, worldLocation(FVector::ZeroVector)
	, worldRotation(FRotator::ZeroRotator)
	, targetPoint(FVector::ZeroVector)
	, focusActor(nullptr)
	, targetDistance(TraceDistance) // Default beam length and pointer distance
{
	// Set this component to be initialized when the game starts, and to be ticked every frame.  You can turn these features
	// off to improve performance if you don't need them.
	PrimaryComponentTick.bCanEverTick = true;

	FString p_fadinglasersystem = TEXT("ParticleSystem'/ViveOpenXRAndroid/Materials/P_FadingLaserSystem.P_FadingLaserSystem'");
	fadingParticleSystemTemplate = Cast<UParticleSystem>(StaticLoadObject(UParticleSystem::StaticClass(), NULL, *p_fadinglasersystem));
	FString m_fadinglasersystem = TEXT("Material'/ViveOpenXRAndroid/Materials/M_FadingLaserSystem.M_FadingLaserSystem'");
	fadingParticleMaterial = Cast<UMaterialInterface>(StaticLoadObject(UMaterialInterface::StaticClass(), NULL, *m_fadinglasersystem));
	//FadingParticle = CreateDefaultSubobject<UParticleSystemComponent>(TEXT("FadingParticle"));
	FadingParticle = NewObject<UParticleSystemComponent>(this, TEXT("FadingParticle"));
	FadingParticle->SetupAttachment(this);

	FString reticle_mesh = TEXT("StaticMesh'/ViveOpenXRAndroid/Shapes/Reticle_Mesh.Reticle_Mesh'");
	spotPointerMesh = Cast<UStaticMesh>(StaticLoadObject(UStaticMesh::StaticClass(), NULL, *reticle_mesh));
	FString m_reticle = TEXT("Material'/ViveOpenXRAndroid/Materials/M_Reticle.M_Reticle'");
	spotPointerMaterial = Cast<UMaterialInterface>(StaticLoadObject(UMaterialInterface::StaticClass(), NULL, *m_reticle));
	//SpotPointer = CreateDefaultSubobject<UStaticMeshComponent>(TEXT("SpotPointer"));
	SpotPointer = NewObject<UStaticMeshComponent>(this, TEXT("SpotPointer"));
	SpotPointer->SetupAttachment(this);

	FString reticle_blue1_mesh = TEXT("StaticMesh'/ViveOpenXRAndroid/Shapes/ReticleBlue1_Mesh.ReticleBlue1_Mesh'");
	spotPointerBlueMesh = Cast<UStaticMesh>(StaticLoadObject(UStaticMesh::StaticClass(), NULL, *reticle_blue1_mesh));
	FString m_reticle_blue1 = TEXT("Material'/ViveOpenXRAndroid/Materials/M_ReticleBlue1.M_ReticleBlue1'");
	spotPointerBlueMaterial = Cast<UMaterialInterface>(StaticLoadObject(UMaterialInterface::StaticClass(), NULL, *m_reticle_blue1));
	//SpotPointerBlue = CreateDefaultSubobject<UStaticMeshComponent>(TEXT("SpotPointerBlue"));
	SpotPointerBlue = NewObject<UStaticMeshComponent>(this, TEXT("SpotPointerBlue"));
	SpotPointerBlue->SetupAttachment(this);

	//WidgetInteraction = CreateDefaultSubobject<UWidgetInteractionComponent>(TEXT("WidgetInteraction"));
	WidgetInteraction = NewObject<UWidgetInteractionComponent>(this, TEXT("WidgetInteraction"));
	WidgetInteraction->SetupAttachment(this);
}

void UWaveHandPointer::CheckLogInterval()
{
	logFrame++;
	logFrame = logFrame % klogFrameCount;
	if (logFrame == 0)
		printIntervalLog = true;
	else
		printIntervalLog = false;
}

#pragma region
void UWaveHandPointer::SetHand(bool isLeft)
{
	IsLeft = isLeft;
}

void UWaveHandPointer::SetWidgetPointerIndex(int32 index)
{
	if (WidgetInteraction)
	{
		WidgetInteraction->PointerIndex = index;
		UE_LOG(LogWaveHandPointer, Log, TEXT("IsLeft %d SetWidgetPointerIndex() %f"), (uint8)m_IsLeft, WidgetInteraction->PointerIndex);
	}
}

void UWaveHandPointer::SetPinchOnThreshold(float strength)
{
	PinchOnThreshold = strength;
}
#pragma endregion Public Interface

void UWaveHandPointer::UpdateCustomizedVariables()
{
	if (m_IsLeft != IsLeft)
	{
		m_IsLeft = IsLeft;
		UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, UpdateCustomizedVariables()"), (uint8)m_IsLeft);
	}

	if (m_PinchOnThreshold != PinchOnThreshold)
	{
		m_PinchOnThreshold = PinchOnThreshold;
		UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, UpdateCustomizedVariables() m_PinchOnThreshold %f")
			, (uint8)m_IsLeft
			, m_PinchOnThreshold);
	}
}

void UWaveHandPointer::ConfigureResources()
{
	if (FadingParticle)
	{
		if (fadingParticleSystemTemplate)
		{
			FadingParticle->SetTemplate(fadingParticleSystemTemplate);
			UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, ConfigureResources() fadingParticleSystemTemplate"), (uint8)m_IsLeft);
		}
		if (fadingParticleMaterial)
		{
			FadingParticle->SetMaterial(fadingParticleMaterialIndex, fadingParticleMaterial);
			UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, ConfigureResources() fadingParticleMaterial"), (uint8)m_IsLeft);
		}
		FVector vec_color = UKismetMathLibrary::Conv_LinearColorToVector(FadingParticleColor);
		FadingParticle->SetVectorParameter(k_InitialColor, vec_color);
		FadingParticle->SetVectorParameter(k_InitialSize, FadingParticleSize);
	}

	if (SpotPointer)
	{
		if (spotPointerMesh)
		{
			SpotPointer->SetStaticMesh(spotPointerMesh);
			UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, ConfigureResources() spotPointerMesh"), (uint8)m_IsLeft);
		}
		if (spotPointerMaterial)
		{
			SpotPointer->SetMaterial(0, spotPointerMaterial);
			UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, ConfigureResources() spotPointerMaterial"), (uint8)m_IsLeft);
		}
		SpotPointer->SetRelativeRotation(SpotRotationOffset);
		SpotPointer->SetRelativeScale3D(SpotScale);
		SpotPointer->SetCollisionEnabled(ECollisionEnabled::Type::NoCollision);

		UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, ConfigureResources() SpotPointer"), (uint8)m_IsLeft);
	}

	if (SpotPointerBlue)
	{
		if (spotPointerBlueMesh)
		{
			SpotPointerBlue->SetStaticMesh(spotPointerBlueMesh);
			UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, ConfigureResources() spotPointerBlueMesh"), (uint8)m_IsLeft);
		}
		if (spotPointerBlueMaterial)
		{
			SpotPointerBlue->SetMaterial(0, spotPointerBlueMaterial);
			UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, ConfigureResources() spotPointerBlueMaterial"), (uint8)m_IsLeft);
		}
		SpotPointerBlue->SetRelativeRotation(SpotRotationOffset);
		SpotPointerBlue->SetRelativeScale3D(SpotScale);
		SpotPointerBlue->SetCollisionEnabled(ECollisionEnabled::Type::NoCollision);
		SpotPointerBlue->SetHiddenInGame(true, true);

		UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, ConfigureResources() SpotPointerBlue"), (uint8)m_IsLeft);
	}

	if (WidgetInteraction)
	{
		WidgetInteraction->VirtualUserIndex = WidgetVirtualUserIndex;
		WidgetInteraction->PointerIndex = WidgetPointerIndex;
		WidgetInteraction->TraceChannel = ECollisionChannel::ECC_Visibility;
		WidgetInteraction->InteractionDistance = TraceDistance;
		WidgetInteraction->InteractionSource = EWidgetInteractionSource::World;
		//WidgetInteraction->bEnableHitTesting = true;
		//WidgetInteraction->bShowDebug = true;
		//WidgetInteraction->DebugColor = FLinearColor::Red;

		UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, ConfigureResources() WidgetInteraction VirtualUserIndex: %d, PointerIndex: %d, TraceChannel: %d, InteractionDistance: %f, InteractionSource: %d")
			, (uint8)m_IsLeft
			, WidgetInteraction->VirtualUserIndex			
			, WidgetInteraction->PointerIndex
			, (int)WidgetInteraction->TraceChannel
			, WidgetInteraction->InteractionDistance
			, (int8)WidgetInteraction->InteractionSource);
	}
}

void UWaveHandPointer::SetIgnoreActors(FCollisionQueryParams& collisionParams)
{
	// Ignore parent actor.
	USceneComponent* parent = GetAttachParent();
	while (parent)
	{
		collisionParams.AddIgnoredActor(parent->GetOwner());
		parent = parent->GetAttachParent();
	}
}

void UWaveHandPointer::BeginPlay()
{
	Super::BeginPlay();

	m_IsLeft = IsLeft;
	m_PinchOnThreshold = PinchOnThreshold;

	ConfigureResources();

	worldLocation = UViveOpenXRAndroidHandInteractionBP::GetPinchPosition(m_IsLeft) + GetComponentLocation();
	FVector pinch_direction = UViveOpenXRAndroidHandInteractionBP::GetPinchRotation(m_IsLeft).GetForwardVector();
	if (pinch_direction.X != 0)
	{
		float value_x = pinch_direction.X > 0 ? pinch_direction.X : -pinch_direction.X;
		pinch_direction = (pinch_direction / value_x); // Normalize
	}
	FVector end_pos = (pinch_direction * TraceDistance) + worldLocation;

	if (FadingParticle)
	{
		FadingParticle->SetBeamSourcePoint(BeamEmitterIndex, worldLocation, BeamSourceIndex);
		FadingParticle->SetBeamTargetPoint(BeamEmitterIndex, end_pos, BeamSourceIndex);
	}

	if (SpotPointer)
	{
		SpotPointer->SetWorldLocation(end_pos);
	}

	if (SpotPointerBlue)
	{
		SpotPointerBlue->SetWorldLocation(end_pos);
	}
}

void UWaveHandPointer::TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction)
{
	Super::TickComponent(DeltaTime, TickType, ThisTickFunction);

	CheckLogInterval();

	UpdateCustomizedVariables();

	if (!IsHandInteractable())
		return;

	tracedPhysics = false;
	tracedGraphics = false;

	worldLocation = UViveOpenXRAndroidHandInteractionBP::GetPinchPosition(m_IsLeft) + GetComponentLocation();
	auto pinch_rotation = UViveOpenXRAndroidHandInteractionBP::GetPinchRotation(m_IsLeft);
	if (printIntervalLog)
	{
		UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, TickComponent() pinch_position (%f, %f, %f)"),
			(uint8)m_IsLeft, worldLocation.X, worldLocation.Y, worldLocation.Z);
		UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, TickComponent() pinch_rotation (%f, %f, %f, %f)"),
			(uint8)m_IsLeft, pinch_rotation.X, pinch_rotation.Y, pinch_rotation.Z, pinch_rotation.W);
	}
	FVector pinch_direction = pinch_rotation.GetForwardVector();
	if (pinch_direction.X != 0)
	{
		float value_x = pinch_direction.X > 0 ? pinch_direction.X : -pinch_direction.X;
		pinch_direction = (pinch_direction / value_x); // Normalize
	}
	FVector end_pos = (pinch_direction * TraceDistance) + worldLocation;
	worldRotation = UKismetMathLibrary::FindLookAtRotation(worldLocation, end_pos);

	targetPoint = (pinch_direction * targetDistance) + worldLocation;

	UpdateButtonStates();

	//FCollisionQueryParams collision_params;
	//SetIgnoreActors(collision_params);
	//tracedPhysics = GetWorld()->LineTraceSingleByChannel(hit, worldLocation, end_pos, ECC_Camera, collision_params);

	FCollisionShape collision_shape;// = FCollisionShape::MakeSphere(500.0f);
	bool casted = GetWorld()->SweepMultiByChannel(outHits, worldLocation, end_pos, FQuat::Identity, ECC_WorldStatic, collision_shape);

	if (casted)
	{
		for (auto& outHit : outHits)
		{
			//AActor* hit_actor = hit.GetActor();
			AActor* hit_actor = outHit.GetActor();

			if (!hit_actor->GetClass()->ImplementsInterface(UWaveHandInteractInterface::StaticClass()))
				continue;
			//LOGD(LogWaveHandPointer, "m_IsLeft %d, TickComponent() Hitting: %s (%s)", (uint8)m_IsLeft, *hit_actor->GetName(), *outHit.ImpactPoint.ToString());

			tracedPhysics = true;
			if (focusActor != hit_actor) // Enter & Exit
			{
				if (focusActor && (focusActor->GetClass()->ImplementsInterface(UWaveHandInteractInterface::StaticClass())))
				{
					// Exit previous actor
					IWaveHandInteractInterface::Execute_OnHandExit(focusActor, GetOwner());
					UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, TickComponent() Exit: %s"), (uint8)m_IsLeft, *focusActor->GetName());
				}
				if (hit_actor && (hit_actor->GetClass()->ImplementsInterface(UWaveHandInteractInterface::StaticClass())))
				{
					// Enter current actor
					IWaveHandInteractInterface::Execute_OnHandEnter(hit_actor, GetOwner());
					UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, TickComponent() Enter: %s"), (uint8)m_IsLeft, *hit_actor->GetName());
				}

				eligibleForClick = false;
				focusActor = hit_actor;
				break; // Use only the 1st hitted actor;
			}
			else // Hover & Click
			{
				if (focusActor && (focusActor->GetClass()->ImplementsInterface(UWaveHandInteractInterface::StaticClass())))
				{
					IWaveHandInteractInterface::Execute_OnHandHover(focusActor, GetOwner());
					if (isDown)
					{
						IWaveHandInteractInterface::Execute_OnHandDown(focusActor, GetOwner());
						UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, TickComponent() Down: %s"), (uint8)m_IsLeft, *hit_actor->GetName());

						eligibleForClick = true;
					}
					if (isUp)
					{
						IWaveHandInteractInterface::Execute_OnHandUp(focusActor, GetOwner());
						UE_LOG(LogWaveHandPointer, Log, TEXT("TickComponent() m_IsLeft %d, Up: %s"), (uint8)m_IsLeft, *hit_actor->GetName());

						if (eligibleForClick)
						{
							IWaveHandInteractInterface::Execute_OnHandClick(focusActor, GetOwner());
							UE_LOG(LogWaveHandPointer, Log, TEXT("TickComponent() m_IsLeft %d, Click: %s"), (uint8)m_IsLeft, *hit_actor->GetName());
						}

						eligibleForClick = false;
					}

					// Change the beam length and pointer distance
					//targetDistance = FVector::Distance(worldLocation, hit.ImpactPoint);
					targetDistance = FVector::Distance(worldLocation, outHit.ImpactPoint);
					targetPoint = (pinch_direction * targetDistance) + worldLocation;
					break; // Use only the 1st hitted actor;
				}
			}
		}
	}
	
	if (!tracedPhysics)
	{
		if (focusActor)
		{
			if (focusActor->GetClass()->ImplementsInterface(UWaveHandInteractInterface::StaticClass()))
			{
				// Exit previous actor
				IWaveHandInteractInterface::Execute_OnHandExit(focusActor, GetOwner());
				UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, TickComponent() Exit: %s"), (uint8)m_IsLeft, *focusActor->GetName());
			}
			focusActor = nullptr;
		}
		eligibleForClick = false;

		if (WidgetInteraction) // Click
		{
			WidgetInteraction->SetWorldLocationAndRotation(worldLocation, worldRotation);
			tracedGraphics = WidgetInteraction->IsOverFocusableWidget();
			if (tracedGraphics)
			{
				UWidgetComponent* widget = WidgetInteraction->GetHoveredWidgetComponent();
				UUserWidget* user_widget = widget->GetUserWidgetObject();
				tracedWidgetPosition = (static_cast<USceneComponent *>(widget))->GetComponentLocation();

				// Change the beam length and pointer distance
				targetDistance = FVector::Distance(worldLocation, tracedWidgetPosition);
				//LOGD(LogWaveHandPointer, "TickComponent() Hovering widget: %s, targetDistance: %f", TCHAR_TO_ANSI(*user_widget->GetFullName()), targetDistance);
				targetPoint = (pinch_direction * targetDistance) + worldLocation;

				if (isUp)
				{
					WidgetInteraction->PressPointerKey(FKey("LeftMouseButton"));
					WidgetInteraction->ReleasePointerKey(FKey("LeftMouseButton"));
					UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d, TickComponent() Click widget: %s"), (uint8)m_IsLeft, *user_widget->GetFullName());
				}
			}
		}
	}

	if (FadingParticle)
	{
		FadingParticle->SetBeamSourcePoint(BeamEmitterIndex, worldLocation, BeamSourceIndex);
		FadingParticle->SetBeamTargetPoint(BeamEmitterIndex, targetPoint, BeamSourceIndex);
	}

	FVector pointer_scale = SpotScale;
	if (targetDistance > k_DefaultPointerDistance)
		pointer_scale = SpotScale * (targetDistance / k_DefaultPointerDistance);
	if (SpotPointer)
	{
		SpotPointer->SetWorldLocation(targetPoint);
		SpotPointer->SetRelativeScale3D(pointer_scale);
	}
	if (SpotPointerBlue)
	{
		SpotPointerBlue->SetWorldLocation(targetPoint);
		SpotPointerBlue->SetRelativeScale3D(pointer_scale);
	}

	ActivateBeamPointer();
}

void UWaveHandPointer::ActivateBeamPointer()
{
	if (!UViveOpenXRAndroidHandInteractionBP::GetPinchTracked(m_IsLeft))
	{
		if (FadingParticle)
			FadingParticle->SetHiddenInGame(true, true);
		if (SpotPointer)
			SpotPointer->SetHiddenInGame(true, true);
		if (SpotPointerBlue)
			SpotPointerBlue->SetHiddenInGame(true, true);
		return;
	}
	else
	{
		if (FadingParticle)
			FadingParticle->SetHiddenInGame(false, true);
	}
	if (!tracedPhysics && !tracedGraphics)
	{
		if (SpotPointer)
			SpotPointer->SetHiddenInGame(true, true);
		if (SpotPointerBlue)
			SpotPointerBlue->SetHiddenInGame(true, true);
		return;
	}
	if (UViveOpenXRAndroidHandInteractionBP::GetPinchTracked(m_IsLeft))
	{
		// Show the blue pointer when pinch on.
		if (SpotPointer)
			SpotPointer->SetHiddenInGame(true, true);
		if (SpotPointerBlue)
			SpotPointerBlue->SetHiddenInGame(false, true);
	}
	else
	{
		// Show the default pointer when pinch off.
		if (SpotPointer)
			SpotPointer->SetHiddenInGame(false, true);
		if (SpotPointerBlue)
			SpotPointerBlue->SetHiddenInGame(true, true);
	}
}
bool UWaveHandPointer::IsHandInteractable()
{
	bool valid_pose = UViveOpenXRAndroidHandInteractionBP::GetPinchTracked(m_IsLeft);

	bool interactable = valid_pose;

	if (printIntervalLog)
	{
		UE_LOG(LogWaveHandPointer, Log, TEXT("IsHandInteractable() m_IsLeft %d, interactable %d, valid_pose %d"),
			(uint8)m_IsLeft, (uint8)interactable, (uint8)valid_pose);
	}

	if (WidgetInteraction)
	{
		if ((interactable && !WidgetInteraction->IsActive()) ||
			(!interactable && WidgetInteraction->IsActive()))
		{
			UE_LOG(LogWaveHandPointer, Log, TEXT("m_IsLeft %d IsHandInteractable() %d"), (uint8)m_IsLeft, (uint8)interactable);
		}
		WidgetInteraction->SetActive(interactable);
	}
	if (SpotPointer)
		SpotPointer->SetHiddenInGame(!interactable, true);
	if (SpotPointerBlue)
		SpotPointerBlue->SetHiddenInGame(!interactable, true);
	if (FadingParticle)
		FadingParticle->SetHiddenInGame(!interactable, true);

	return interactable;
}

void UWaveHandPointer::UpdateButtonStates()
{
	float strength = UViveOpenXRAndroidHandInteractionBP::GetPinchStrength(m_IsLeft);
	if (printIntervalLog)
	{
		UE_LOG(LogWaveHandPointer, Log, TEXT("UpdateButtonStates() m_IsLeft %d, strength %f"), (uint8)m_IsLeft, strength);
	}
	bool pressed = strength > m_PinchOnThreshold;

	isDown = false;
	isUp = false;

	bool down = (pressed && !pressState);
	bool hold = (pressed && pressState);
	bool up = (!pressed && pressState);

	if (!down && hold) {
		// ToDo: drag
	}
	else if (down && !eligibleForClick) {
		isDown = true;
	}
	else if (up) {
		isUp = true;
	}

	pressState = pressed;
}
